@@ -31,11 +31,11 @@ from gitlab.exceptions import *  # noqa
 from gitlab import utils  # noqa
 
 __title__ = 'python-gitlab'
-__version__ = '1.6.0'
+__version__ = '1.8.0'
 __author__ = 'Gauvain Pocentek'
-__email__ = 'gauvain@pocentek.net'
+__email__ = 'gauvainpocentek@gmail.com'
 __license__ = 'LGPL3'
-__copyright__ = 'Copyright 2013-2018 Gauvain Pocentek'
+__copyright__ = 'Copyright 2013-2019 Gauvain Pocentek'
 
 warnings.filterwarnings('default', category=DeprecationWarning,
                         module='^gitlab')
@@ -411,7 +411,7 @@ class Gitlab(object):
                 if item.status_code not in (301, 302):
                     continue
                 # GET methods can be redirected without issue
-                if result.request.method == 'GET':
+                if item.request.method == 'GET':
                     continue
                 # Did we end-up with an https:// URL?
                 location = item.headers.get('Location', None)
@@ -445,7 +445,20 @@ class Gitlab(object):
 
         params = {}
         utils.copy_dict(params, query_data)
-        utils.copy_dict(params, kwargs)
+
+        # Deal with kwargs: by default a user uses kwargs to send data to the
+        # gitlab server, but this generates problems (python keyword conflicts
+        # and python-gitlab/gitlab conflicts).
+        # So we provide a `query_parameters` key: if it's there we use its dict
+        # value as arguments for the gitlab server, and ignore the other
+        # arguments, except pagination ones (per_page and page)
+        if 'query_parameters' in kwargs:
+            utils.copy_dict(params, kwargs['query_parameters'])
+            for arg in ('per_page', 'page'):
+                if arg in kwargs:
+                    params[arg] = kwargs[arg]
+        else:
+            utils.copy_dict(params, kwargs)
 
         opts = self._get_session_opts(content_type='application/json')
 
@@ -477,6 +490,10 @@ class Gitlab(object):
         # obey the rate limit by default
         obey_rate_limit = kwargs.get("obey_rate_limit", True)
 
+        # set max_retries to 10 by default, disable by setting it to -1
+        max_retries = kwargs.get("max_retries", 10)
+        cur_retries = 0
+
         while True:
             result = self.session.send(prepped, timeout=timeout, **settings)
 
@@ -486,14 +503,22 @@ class Gitlab(object):
                 return result
 
             if 429 == result.status_code and obey_rate_limit:
-                wait_time = int(result.headers["Retry-After"])
-                time.sleep(wait_time)
-                continue
+                if max_retries == -1 or cur_retries < max_retries:
+                    wait_time = 2 ** cur_retries * 0.1
+                    if "Retry-After" in result.headers:
+                        wait_time = int(result.headers["Retry-After"])
+                    cur_retries += 1
+                    time.sleep(wait_time)
+                    continue
 
+            error_message = result.content
             try:
-                error_message = result.json()['message']
+                error_json = result.json()
+                for k in ('message', 'error'):
+                    if k in error_json:
+                        error_message = error_json[k]
             except (KeyError, ValueError, TypeError):
-                error_message = result.content
+                pass
 
             if result.status_code == 401:
                 raise GitlabAuthenticationError(
@@ -505,7 +530,8 @@ class Gitlab(object):
                                   error_message=error_message,
                                   response_body=result.content)
 
-    def http_get(self, path, query_data={}, streamed=False, **kwargs):
+    def http_get(self, path, query_data={}, streamed=False, raw=False,
+                 **kwargs):
         """Make a GET request to the Gitlab server.
 
         Args:
@@ -513,6 +539,7 @@ class Gitlab(object):
                         'http://whatever/v4/api/projecs')
             query_data (dict): Data to send as query parameters
             streamed (bool): Whether the data should be streamed
+            raw (bool): If True do not try to parse the output as json
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
@@ -526,8 +553,10 @@ class Gitlab(object):
         """
         result = self.http_request('get', path, query_data=query_data,
                                    streamed=streamed, **kwargs)
-        if (result.headers['Content-Type'] == 'application/json' and
-           not streamed):
+
+        if (result.headers['Content-Type'] == 'application/json'
+           and not streamed
+           and not raw):
             try:
                 return result.json()
             except Exception:
@@ -561,7 +590,7 @@ class Gitlab(object):
         # In case we want to change the default behavior at some point
         as_list = True if as_list is None else as_list
 
-        get_all = kwargs.get('all', False)
+        get_all = kwargs.pop('all', False)
         url = self._build_url(path)
 
         if get_all is True:
